@@ -30,10 +30,10 @@
 #'
 #' @return A vector (or matrix) consist of item or test information.
 #'
-#' @include Item-class.R
-#' @include Item-class-methods.R
-#' @include Itempool-class.R
-#' @include Itempool-class-methods.R
+#' @include item-class.R
+#' @include item-class-methods.R
+#' @include itempool-class.R
+#' @include itempool-class-methods.R
 #'
 #' @author Emre Gonulates
 #'
@@ -51,7 +51,6 @@ setGeneric("info", function(ip, theta, tif = FALSE, observed = FALSE,
 #' @rdname info
 #'
 #' @examples
-#' info(ip = generate_item(model = "Rasch"), theta = rnorm(1))
 #' info(ip = generate_item(model = "1PL"), theta = rnorm(1))
 #' info(ip = generate_item(model = "2PL"), theta = rnorm(1))
 #' info(ip = generate_item(model = "3PL"), theta = rnorm(1))
@@ -63,34 +62,191 @@ setGeneric("info", function(ip, theta, tif = FALSE, observed = FALSE,
 #'
 setMethod(
   f = "info", signature = c(ip = "Item"),
-  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL){
-    # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (all(ip$model %in%
-              c(names(Pmodels)[
-                sapply(Pmodels, function(x) x$model_family == 'UIRT')],
-                "GRM", "GPCM", "PCM", "GPCM2"))) {
-      return(info_item_cpp(theta = theta, item = ip, observed = observed,
-                           resp = resp))
-    } else stop("This model has not been implemented in 'info()' function.",
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    stop("This model has not been implemented in 'info()' function.",
                 call. = FALSE)
   }
 )
 
 
+###############################################################################@
+############################# info (Rasch) #####################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "Rasch"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "Rasch"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+###############################################################################@
+############################# info (1PL) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "1PL"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "1PL"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+###############################################################################@
+############################# info (2PL) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "2PL"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "2PL"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+###############################################################################@
+############################# info (3PL) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "3PL"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "3PL"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+
+###############################################################################@
+############################# info (4PL) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "4PL"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "4PL"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+
+###############################################################################@
+############################# info (PCM) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "PCM"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "PCM"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+
+###############################################################################@
+############################# info (GRM) #######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "GRM"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "GRM"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+
+###############################################################################@
+############################# info (GPCM) ######################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "GPCM"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "GPCM"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+###############################################################################@
+############################# info (GPCM2) #####################################
+###############################################################################@
+#' @export
+#'
+#' @rdname info
+#'
+#' @examples
+#' info(ip = generate_item(model = "GPCM2"), theta = rnorm(1))
+#'
+setMethod(
+  f = "info", signature = c(ip = "GPCM2"),
+  function(ip, theta, tif = FALSE, observed = FALSE, resp = NULL) {
+    info_item_cpp(theta = theta, item = ip, observed = observed, resp = resp)
+  }
+)
+
+
+
+
 ############################# .info_itempool ##############################@###
 .info_itempool <-  function(ip, theta, tif = FALSE, observed = FALSE,
                              resp = NULL) {
   if (inherits(ip, "Testlet")) ip <- ip@item_list
   # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-  acceptable_models <- c(
-    names(Pmodels)[sapply(Pmodels, function(x) x$model_family == 'UIRT')],
-    "GRM", "GPCM", "PCM", "GPCM2")
+  acceptable_models <- c(UNIDIM_DICHO_MODELS, UNIDIM_POLY_MODELS)
   # All items should be acceptable
   test <- FALSE
 
   # All standalone Item models should be valid
-  if ("Item" %in% sapply(ip, class))
-    test <- test | all(ip[sapply(ip, class)  == "Item"]$model %in%
+  if (any(sapply(ip, inherits, "Item")))
+    test <- test | all(ip[sapply(ip, inherits, "Item")]$model %in%
                          acceptable_models)
   # Check whether Testlet items are valid
   if ("Testlet" %in% sapply(ip, class))
@@ -99,21 +255,33 @@ setMethod(
                     function(x) x@item_list$model)) %in% acceptable_models)
   # Check whether resp is acceptable
   if (!is.null(resp)) {
+    if (is(resp, "Response")) resp <- response_set(resp)
+    if (is(resp, "Response_set")) {
+      if (test) {
+        return(info_response_set_cpp(
+          theta = theta, ip = ip, resp_set = resp, tif = tif,
+          observed = observed))
+      } else stop("This model is not implemented in 'info()' function.")
+    }
+
     if (!is.numeric(resp))
       stop("Invalid response vector. It should be numeric.")
-    if (!is.matrix(resp)) resp <- matrix(resp, nrow = 1)
-    ip_size <- get_itempool_size(ip)
-    if (nrow(resp) != length(theta) || ncol(resp) != ip_size["items"])
+    if (!is.matrix(resp))
+      resp <- matrix(resp, nrow = 1)
+    if (nrow(resp) != length(theta) || ncol(resp) != ip$n$items)
       stop("Invalid response vector/matrix. The item pool size and response ",
            "size do not match.")
   }
   if (test) {
     return(info_itempool_cpp(theta = theta, ip = ip, tif = tif,
-                              observed = observed, resp = resp))
+                             observed = observed, resp = resp))
   } else stop("This model is not implemented in 'info()' function.")
 }
 
 
+
+
+
 ###############################################################################@
 ############################# info (Itempool) #################################
 ###############################################################################@
@@ -223,10 +391,10 @@ setMethod(
 #' @return A vector (or matrix) consist of item or test Kullback-Leibler
 #' information.
 #'
-#' @include Item-class.R
-#' @include Itempool-class.R
-#' @include Item-class-methods.R
-#' @include Itempool-class-methods.R
+#' @include item-class.R
+#' @include itempool-class.R
+#' @include item-class-methods.R
+#' @include itempool-class-methods.R
 #'
 #' @author Emre Gonulates
 #'
@@ -272,15 +440,14 @@ setMethod(
     #       phi = 45, theta = -45, ticktype = "detailed")
     #
 
-    # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (ip$model %in%
-          names(Pmodels)[sapply(Pmodels, function(x) x$model_family == 'UIRT')])
+    # If the Model is one of the following: "1PM" "2PM" "3PM" "4PM"
+    if (ip$model %in% UNIDIM_DICHO_MODELS)
     {
       if ((length(trueTheta) > 1) || (length(theta) > 1))
       {
-        pTrue <- matrix(prob(ip = ip, theta = trueTheta),
+        pTrue <- matrix(prob(ip = ip, theta = trueTheta)[, 2],
                         nrow = length(trueTheta), ncol = length(theta))
-        p <- matrix(prob(ip = ip, theta = theta),
+        p <- matrix(prob(ip = ip, theta = theta)[, 2],
                     nrow = length(trueTheta), ncol = length(theta), byrow = T)
       } else {
         pTrue <- prob(ip = ip, theta = trueTheta)
