@@ -28,7 +28,7 @@
 #'           of \code{ip2} to the standard deviation of \code{ip1}.}
 #'         \item{\code{output$a$robust_z}}{Robust-z statistic values for each
 #'           item discrimination parameter.}
-#'         \item{\code{output$a$unstable}}{Item id's which were flagged if
+#'         \item{\code{output$a$unstable}}{Item ID's which were flagged if
 #'           robust z statistic value for a parameters is larger than the
 #'           absolute value of  the critical value
 #'           (i.e. \code{qnorm(1-alphe/2)}).}
@@ -36,7 +36,7 @@
 #'           item difficulty or threshold parameter. If an item has threshold
 #'           parameters, robust z statistic will be calculated for each
 #'           threshold.}
-#'         \item{\code{output$b$unstable}}{Item id's which were flagged if
+#'         \item{\code{output$b$unstable}}{Item ID's which were flagged if
 #'           robust z statistic for difficulty/threshold parameters are larger
 #'           than the absolute value of the critical value (i.e.
 #'           \code{qnorm(1-alphe/2)}).}
@@ -53,7 +53,7 @@
 #' Huynh, Huynh and Meyer, Patrick (2010) "Use of Robust z in Detecting
 #' Unstable Items in Item Response Theory Models,"
 #' \emph{Practical Assessment, Research, and Evaluation}: Vol. 15 , Article 2.
-#' DOI: \url{https://doi.org/10.7275/ycx6-e864}
+#' DOI:  <doi:10.7275/ycx6-e864>
 #' Available at: \url{https://scholarworks.umass.edu/pare/vol15/iss1/2/}
 #'
 #' @examples
@@ -126,7 +126,7 @@ ipd <- function(ip1, ip2, method = "robust-z", anchor_item_ids = NULL,
     output$a$sd_ratio <- stats::sd(a2)/stats::sd(a1)
 
     cv <- stats::qnorm(1 - alpha/2) # two tailed critical value
-    output$a$unstable <- ip1$id[abs(output$a$robust_z) >= cv]
+    output$a$unstable <- ip1$item_id[abs(output$a$robust_z) >= cv]
 
     # Select only stable items to calculate the linking constant
     A <- exp(mean(a_diff[abs(output$a$robust_z) < cv]))
@@ -136,14 +136,14 @@ ipd <- function(ip1, ip2, method = "robust-z", anchor_item_ids = NULL,
 
     b1 <- b2 <- c()
     for (i in seq_along(ip1_list)) {
-      temp_b <- ip1_list[[i]]@parameters$b
+      temp_b <- ip1_list[[i]]$b
       b1 <- c(b1, stats::setNames(
-        temp_b, paste0(ip1_list[[i]]$id, if (length(temp_b) > 1)
+        temp_b, paste0(ip1_list[[i]]$item_id, if (length(temp_b) > 1)
           paste0(".", 1:length(temp_b)) else NULL)))
 
-      temp_b <- ip2_list[[i]]@parameters$b
+      temp_b <- ip2_list[[i]]$b
       b2 <- c(b2, stats::setNames(
-        temp_b, paste0(ip1_list[[i]]$id, if (length(temp_b) > 1)
+        temp_b, paste0(ip1_list[[i]]$item_id, if (length(temp_b) > 1)
           paste0(".", 1:length(temp_b)) else NULL)))
 
     }
