@@ -19,8 +19,8 @@
 #'   multiple possible responses, probability of each response category will be
 #'   returned.
 #'
-#' @param ip An \code{\link{Item-class}} or an \code{\link{Itempool-class}}
-#'   object containing the item parameters.
+#' @param ip An \code{\link{Item-class}}, or an \code{\link{Itempool-class}} or
+#'  \code{\link{Testlet-class}}  object containing the item parameters.
 #' @param theta An object containing the ability parameters.
 #' @param derivative Whether to calculate the first or second derivative of
 #'   probability of a response.
@@ -30,21 +30,17 @@
 #'     \item{\code{1}}{Calculate the first derivative.}
 #'     \item{\code{2}}{Calculate the second derivative.}
 #'   }
-#' @param expected_value For each possible response value, the probability
-#'   of that response is calculated and summed to get the expected value at
-#'   a \code{theta} value. Default value is \code{FALSE}.
 #'
-#' @return Item probabilities at given theta will be returned. If
-#'   \code{expected_value} is \code{TRUE}, the expected value(s) of item or
-#'   item pool at a given \code{theta} value will be returned.
+#' @return Item probabilities at given theta will be returned.
 #'
-#' @include Item-class.R
-#' @include Itempool-class.R
+#'
+#' @include item-class.R
+#' @include itempool-class.R
 #'
 #' @author Emre Gonulates
 #'
-setGeneric("prob", function(ip, theta, derivative = 0, expected_value = FALSE)
-  {standardGeneric ("prob")})
+setGeneric("prob", function(ip, theta, derivative = 0)
+  {standardGeneric("prob")})
 
 
 ###############################################################################@
@@ -54,6 +50,126 @@ setGeneric("prob", function(ip, theta, derivative = 0, expected_value = FALSE)
 #' @useDynLib irt
 #' @rdname prob
 #'
+setMethod(
+  f = "prob", signature = c(ip = "Item"),
+  function(ip, theta, derivative = 0){
+    stop("This model has not been implemented in 'prob()' function yet.")
+  }
+)
+
+
+############################# .prob_4pm_item ###################################
+#' This function is general method for all 4PM models
+#' @noRd
+#'
+.prob_4pm_item <- function(ip, theta, derivative = 0) {
+  if (!derivative %in% 0:2)
+    stop("Invalid 'derivative'. 'derivative' value can be 0, 1 or 2.")
+
+  p <- prob_4pm_item_cpp(theta = theta, item = ip, derivative = derivative)
+  if (derivative == 0) q <- 1 - p else q <- -p
+  return(matrix(c(q, p), ncol = 2, dimnames = list(names(theta), 0:1)))
+  # return(stats::setNames(prob_4pm_item_cpp(
+  #   theta = theta, item = ip, derivative = derivative), ip@item_id))
+}
+
+
+############################# .prob_poly_item ##################################
+#' This function is general method for GRM, PCM, GPCM, GPCM2 models
+#' @noRd
+#'
+.prob_poly_item <- function(ip, theta, derivative = 0) {
+  # Expected value can only be calculated when derivative = 0
+  if (!derivative %in% 0:2)
+    stop("Invalid 'derivative'. 'derivative' value can be 0, 1 or 2.")
+
+  result <- sapply(theta, prob_poly_bare_cpp, item = ip,
+                   derivative = derivative, resp = -9, expected_value = FALSE)
+  result <- t(result)
+  dimnames(result) <- list(names(theta), paste0(0:(ncol(result) - 1)))
+  return(result)
+}
+
+
+###############################################################################@
+############################# prob (Rasch) #####################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "Rasch")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "Rasch"),
+  function(ip, theta, derivative = 0) {
+    .prob_4pm_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+###############################################################################@
+############################# prob (1PL) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "1PL")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "1PL"),
+  function(ip, theta, derivative = 0){
+    .prob_4pm_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+
+
+###############################################################################@
+############################# prob (2PL) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
 #' @examples
 #' theta <- rnorm(1)
 #' item1 <- generate_item(model = "2PL")
@@ -67,6 +183,197 @@ setGeneric("prob", function(ip, theta, derivative = 0, expected_value = FALSE)
 #' # Second derivative of probability of correct response:
 #' prob(item1, theta, derivative = 2)
 #'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "2PL"),
+  function(ip, theta, derivative = 0){
+    .prob_4pm_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+
+###############################################################################@
+############################# prob (3PL) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "3PL")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "3PL"),
+  function(ip, theta, derivative = 0){
+    .prob_4pm_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+###############################################################################@
+############################# prob (4PL) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "4PL")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "4PL"),
+  function(ip, theta, derivative = 0){
+    .prob_4pm_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+
+###############################################################################@
+############################# prob (GRM) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "GRM")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#'
+#'
+#' item4 <- generate_item(model = "GRM", n_categories = 5)
+#' prob(item4, theta)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "GRM"),
+  function(ip, theta, derivative = 0){
+    .prob_poly_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+###############################################################################@
+############################# prob (PCM) #######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' # Partial Credit Model
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "PCM")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
+#'
+#' item3 <- generate_item(model = "GPCM2", n_categories = 3)
+#' prob(item3, theta)
+#'
+setMethod(
+  f = "prob", signature = c(ip = "PCM"),
+  function(ip, theta, derivative = 0){
+    .prob_poly_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+
+
+###############################################################################@
+############################# prob (GPCM) ######################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
+#'
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "GPCM")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
+#'
+#'
+#' # Multiple theta values
+#' theta_n <- rnorm(5)
+#'
+#' prob(item1, theta_n)
+#' prob(item1, theta_n, derivative = 1)
+#' prob(item1, theta_n, derivative = 2)
+#'
 #' # Probability of each response category for Generalized Partial Credit Model
 #' item2 <- generate_item(model = "GPCM", n_categories = 4)
 #' prob(item2, theta)
@@ -77,17 +384,35 @@ setGeneric("prob", function(ip, theta, derivative = 0, expected_value = FALSE)
 #' # Second derivative of each response category
 #' prob(item2, theta, derivative = 2)
 #'
-#' # Expected score for a subject with a given theta value
-#' prob(item2, theta, expected_value = TRUE)
+setMethod(
+  f = "prob", signature = c(ip = "GPCM"),
+  function(ip, theta, derivative = 0){
+    .prob_poly_item(ip = ip, theta = theta, derivative = derivative)
+  }
+)
+
+
+
+###############################################################################@
+############################# prob (GPCM2) #####################################
+###############################################################################@
+#' @export
+#' @useDynLib irt
+#' @rdname prob
 #'
-#' # Probability of each response category for Reparametrized Generalized
-#' # Partial Credit Model
-#' item3 <- generate_item(model = "GPCM2", n_categories = 3)
-#' prob(item3, theta)
+#' @examples
+#' theta <- rnorm(1)
+#' item1 <- generate_item(model = "GPCM2")
+#'
+#' # Probability of correct response
+#' prob(item1, theta)
+#'
+#' # First derivative of probability of correct response:
+#' prob(item1, theta, derivative = 1)
+#'
+#' # Second derivative of probability of correct response:
+#' prob(item1, theta, derivative = 2)
 #'
-#' # Probability of each response category for Graded Response Model
-#' item4 <- generate_item(model = "GRM", n_categories = 5)
-#' prob(item4, theta)
 #'
 #' # Multiple theta values
 #' theta_n <- rnorm(5)
@@ -96,44 +421,17 @@ setGeneric("prob", function(ip, theta, derivative = 0, expected_value = FALSE)
 #' prob(item1, theta_n, derivative = 1)
 #' prob(item1, theta_n, derivative = 2)
 #'
-#' prob(item2, theta_n)
-#' prob(item2, theta_n, derivative = 1)
-#' prob(item2, theta_n, derivative = 2)
-#'
 setMethod(
-  f = "prob", signature = c(ip = "Item"),
-  function(ip, theta, derivative = 0, expected_value = FALSE){
-    # Expected value can only be calculated when derivative = 0
-    if (expected_value && derivative != 0)
-      stop("'expected_value' can only be calculated for 'derivative = 0'.")
-    # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (ip@model %in%
-        names(Pmodels)[sapply(Pmodels, function(x) x$model_family == "UIRT")]) {
-      return(prob_4pm_item_cpp(theta = theta, item = ip,
-                               derivative = derivative))
-    } else if (ip@model %in%
-        names(Pmodels)[sapply(Pmodels, function(x) x$model_family == "PIRT")]) {
-      result <-  sapply(theta, prob_poly_bare_cpp,  item = ip,
-                        derivative = derivative,
-                        expected_value = expected_value)
-      if (expected_value) {
-        result <- c(result)
-      } else {
-        result <- t(result)
-        colnames(result) <- paste0("category.", 0:(ncol(result)-1))
-      }
-      return(result)
-    } else if (ip@model %in% names(Pmodels)[
-      sapply(Pmodels, function(x) x$model_family == "MIRT")]) {
-      return(prob_mirt_item_cpp(theta = theta, item = ip,
-                                derivative = derivative))
-    } else stop("This model has not been implemented in 'prob()' function yet.")
+  f = "prob", signature = c(ip = "GPCM2"),
+  function(ip, theta, derivative = 0){
+    .prob_poly_item(ip = ip, theta = theta, derivative = derivative)
   }
 )
 
 
+
 ###############################################################################@
-############################# prob (Itempool) #################################
+############################# prob (Itempool) ##################################
 ###############################################################################@
 #' @export
 #'
@@ -154,7 +452,7 @@ setMethod(
 #' prob(ip, theta, derivative = 2)
 #'
 #' # Multiple theta
-#' theta_n <- rnorm(5)
+#' theta_n <- rnorm(3)
 #' prob(ip, theta_n)
 #' prob(ip, theta_n, derivative = 1)
 #' prob(ip, theta_n, derivative = 2)
@@ -170,9 +468,6 @@ setMethod(
 #' # Second derivative of each response category
 #' prob(ip, theta, derivative = 2)
 #'
-#' # Expected score for a subject with a given theta value for each item
-#' prob(ip, theta, expected_value = TRUE)
-#'
 #' # Probability of a mixture of items models
 #' ip <- generate_ip(model = c("GPCM", "2PL", "3PL", "GPCM"),
 #'                   n_categories = c(4, 2, 2, 3))
@@ -180,35 +475,33 @@ setMethod(
 #'
 setMethod(
   f = "prob", signature = c(ip = "Itempool"),
-  function(ip,  theta, derivative = 0, expected_value = FALSE){
+  function(ip,  theta, derivative = 0){
     # Expected value can only be calculated when derivative = 0
-    if (expected_value && derivative != 0)
-      stop("'expected_value' can only be calculated for 'derivative = 0'.")
-    # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (all(ip$model %in%
-          names(Pmodels)[sapply(Pmodels,
-                                function(x) x$model_family == "UIRT")])) {
-      return(prob_4pm_itempool_cpp(theta = theta, ip = ip,
-                                   derivative = derivative))
-    } else if (all(ip$model %in% names(Pmodels)[
-      sapply(Pmodels, function(x) x$model_family == "MIRT")])) {
-      return(prob_mirt_itempool_cpp(theta = theta, ip = ip,
-                                    derivative = derivative))
-    } else if (length(theta) == 1) {
-      result <- prob_bare_itempool_cpp(theta, ip, derivative, expected_value)
-      dimnames(result) <- list(ip$resp_id, paste0(0:(ncol(result)-1)))
-      if (expected_value) result <- result[, 1]
-    } else if (expected_value) { # multiple theta and expected_value = TRUE
-                                 # mix of dichotomous and polytomous items
-      result <- t(sapply(theta, prob_bare_itempool_cpp, ip = ip,
-                         derivative = derivative,
-                         expected_value = expected_value))
-      dimnames(result) <- list(names(theta), ip$resp_id)
-    } else
-      stop("prob() function cannot be run on item sets that are composed of ",
-           "unidimensional and multidimensional items or item sets with ",
-           "polytomous items with more than one theta. This model has not ",
-           "been implemented in 'prob()' function yet.")
+    if (!derivative %in% 0:2)
+      stop("Invalid 'derivative'. 'derivative' value can be 0, 1 or 2.")
+
+    result <- lapply(theta, prob_bare_itempool_cpp, ip = ip,
+                     derivative = derivative, expected_value = FALSE)
+    names(result) <- names(theta)
+    if (length(result) == 1) result <- result[[1]]
+    # # If the Model is one of the following: "1PM" "irt2PM" "irt3PM" "irt4PM"
+    # if (all(ip$model %in% UNIDIM_DICHO_MODELS)) {
+    #   result <- prob_4pm_itempool_cpp(theta = theta, ip = ip,
+    #                                   derivative = derivative)
+    #   dimnames(result) <- list(names(theta), ip$resp_id)
+    # } else if (all(ip$model %in% names(PMODELS)[
+    #   sapply(PMODELS, function(x) x$model_family == "MIRT")])) {
+    #   return(prob_mirt_itempool_cpp(theta = theta, ip = ip,
+    #                                 derivative = derivative))
+    # } else if (length(theta) == 1) {
+    #
+    #   # result <- prob_bare_itempool_cpp(theta, ip, derivative, FALSE)
+    #   # dimnames(result) <- list(ip$resp_id, paste0(0:(ncol(result) - 1)))
+    # } else
+    #   stop("prob() function cannot be run on item sets that are composed of ",
+    #        "unidimensional and multidimensional items or item sets with ",
+    #        "polytomous items with more than one theta. This model has not ",
+    #        "been implemented in 'prob()' function yet.")
     return(result)
   }
 )
@@ -236,9 +529,8 @@ setMethod(
 #'
 setMethod(
   f = "prob", signature = c(ip = "Testlet"),
-  function(ip,  theta, derivative = 0, expected_value = FALSE){
-    return(prob(ip = ip@item_list, theta = theta, derivative = derivative,
-                expected_value = expected_value))
+  function(ip,  theta, derivative = 0){
+    return(prob(ip = ip@item_list, theta = theta, derivative = derivative))
   }
 )
 
@@ -251,13 +543,11 @@ setMethod(
 #' @rdname prob
 setMethod(
   f = "prob", signature = c(ip = "numMatDfListChar"),
-  function(ip, theta, derivative = 0, expected_value = FALSE) {
+  function(ip, theta, derivative = 0) {
     if (inherits(ip, "numeric")) {
-      return(prob(ip = itempool(ip), theta = theta, derivative = derivative,
-                  expected_value = expected_value))
+      return(prob(ip = itempool(ip), theta = theta, derivative = derivative))
     } else if (inherits(ip, c("data.frame", "matrix", "list"))) {
-      return(prob(ip = itempool(ip), theta = theta, derivative = derivative,
-                  expected_value = expected_value))
+      return(prob(ip = itempool(ip), theta = theta, derivative = derivative))
     } else {
       stop("Cannot convert object to an 'Item' or an 'Itempool' object. ",
            "Please provide a valid 'Item' or 'Itempool' object using either ",
